
from asf_snow.utils.combine_multiple_statistic_results import *

file="/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_202011_202104_500_24h_12h_s1_average_y.txt"

draw_snow_difference_histgram(file)


